/**
 * ##% Copyright (C) 2008 Code Lutin, Tony Chemit
 * This program is free software; you
 * can redistribute it and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. This program is
 * distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details. You
 * should have received a copy of the GNU General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 59 Temple Place
 * - Suite 330, Boston, MA 02111-1307, USA. 
 * ##%
 */
package org.nuiton.jaxx.action;

/**
 * Contract to be realized by a provider of Actions.
 * <p/>
 * A provider of actions is based on a certain type of {@link MyAbstractAction} (<code>A</code>) with an accessor
 * {@link #getBaseClass()} and deliver some implementations of such actions indexed by their logical names via the
 * method {#link #getClasses()}.
 *
 * @param <A> type of action boxed
 * @author chemit
 * @see MyAbstractAction
 */
public interface ActionProvider<A extends MyAbstractAction> {

    /** @return the base classe of provided actions */
    Class<A> getBaseClass();

    /** @return the provided actions classes */
    java.util.Map<String, Class<? extends A>> getClasses();

}
